@@ -17,6 +17,7 @@
  */
 
 #include "SearchWidget.h"
+#include "ui_SearchHelpWidget.h"
 #include "ui_SearchWidget.h"
 
 #include <QKeyEvent>
@@ -26,28 +27,40 @@
 
 #include "core/Config.h"
 #include "core/FilePath.h"
+#include "gui/widgets/PopupHelpWidget.h"
 
 SearchWidget::SearchWidget(QWidget* parent)
     : QWidget(parent)
     , m_ui(new Ui::SearchWidget())
+    , m_searchTimer(new QTimer(this))
+    , m_clearSearchTimer(new QTimer(this))
 {
     m_ui->setupUi(this);
 
-    m_searchTimer = new QTimer(this);
+    m_helpWidget = new PopupHelpWidget(m_ui->searchEdit);
+    m_helpWidget->setOffset(QPoint(0, 1));
+    Ui::SearchHelpWidget helpUi;
+    helpUi.setupUi(m_helpWidget);
+
     m_searchTimer->setSingleShot(true);
+    m_clearSearchTimer->setSingleShot(true);
 
     connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), SLOT(startSearchTimer()));
     connect(m_ui->clearIcon, SIGNAL(triggered(bool)), m_ui->searchEdit, SLOT(clear()));
+    connect(m_ui->helpIcon, SIGNAL(triggered()), SLOT(toggleHelp()));
     connect(m_searchTimer, SIGNAL(timeout()), this, SLOT(startSearch()));
+    connect(m_clearSearchTimer, SIGNAL(timeout()), m_ui->searchEdit, SLOT(clear()));
     connect(this, SIGNAL(escapePressed()), m_ui->searchEdit, SLOT(clear()));
 
-    new QShortcut(Qt::CTRL + Qt::Key_F, this, SLOT(searchFocus()), nullptr, Qt::ApplicationShortcut);
+    new QShortcut(QKeySequence::Find, this, SLOT(searchFocus()), nullptr, Qt::ApplicationShortcut);
     new QShortcut(Qt::Key_Escape, m_ui->searchEdit, SLOT(clear()), nullptr, Qt::ApplicationShortcut);
 
+    m_ui->searchEdit->setPlaceholderText(tr("Search (%1)...", "Search placeholder text, %1 is the keyboard shortcut")
+                                             .arg(QKeySequence(QKeySequence::Find).toString(QKeySequence::NativeText)));
     m_ui->searchEdit->installEventFilter(this);
 
     QMenu* searchMenu = new QMenu();
-    m_actionCaseSensitive = searchMenu->addAction(tr("Case Sensitive"), this, SLOT(updateCaseSensitive()));
+    m_actionCaseSensitive = searchMenu->addAction(tr("Case sensitive"), this, SLOT(updateCaseSensitive()));
     m_actionCaseSensitive->setObjectName("actionSearchCaseSensitive");
     m_actionCaseSensitive->setCheckable(true);
 
@@ -60,6 +73,9 @@ SearchWidget::SearchWidget(QWidget* parent)
     m_ui->searchIcon->setMenu(searchMenu);
     m_ui->searchEdit->addAction(m_ui->searchIcon, QLineEdit::LeadingPosition);
 
+    m_ui->helpIcon->setIcon(filePath()->icon("actions", "system-help"));
+    m_ui->searchEdit->addAction(m_ui->helpIcon, QLineEdit::TrailingPosition);
+
     m_ui->clearIcon->setIcon(filePath()->icon("actions", "edit-clear-locationbar-rtl"));
     m_ui->clearIcon->setVisible(false);
     m_ui->searchEdit->addAction(m_ui->clearIcon, QLineEdit::TrailingPosition);
@@ -81,13 +97,6 @@ bool SearchWidget::eventFilter(QObject* obj, QEvent* event)
         if (keyEvent->key() == Qt::Key_Escape) {
             emit escapePressed();
             return true;
-        } else if (keyEvent->matches(QKeySequence::Copy)) {
-            // If Control+C is pressed in the search edit when no text
-            // is selected, copy the password of the current entry
-            if (!m_ui->searchEdit->hasSelectedText()) {
-                emit copyPressed();
-                return true;
-            }
         } else if (keyEvent->matches(QKeySequence::MoveToNextLine)) {
             if (m_ui->searchEdit->cursorPosition() == m_ui->searchEdit->text().length()) {
                 // If down is pressed at EOL, move the focus to the entry view
@@ -99,18 +108,25 @@ bool SearchWidget::eventFilter(QObject* obj, QEvent* event)
                 return true;
             }
         }
+    } else if (event->type() == QEvent::FocusOut) {
+        // Auto-clear search after 5 minutes
+        m_clearSearchTimer->start(300000);
+    } else if (event->type() == QEvent::FocusIn) {
+        m_clearSearchTimer->stop();
     }
 
-    return QObject::eventFilter(obj, event);
+    return QWidget::eventFilter(obj, event);
 }
 
 void SearchWidget::connectSignals(SignalMultiplexer& mx)
 {
+    // Connects basically only to the current DatabaseWidget, but allows to switch between instances!
     mx.connect(this, SIGNAL(search(QString)), SLOT(search(QString)));
     mx.connect(this, SIGNAL(caseSensitiveChanged(bool)), SLOT(setSearchCaseSensitive(bool)));
     mx.connect(this, SIGNAL(limitGroupChanged(bool)), SLOT(setSearchLimitGroup(bool)));
     mx.connect(this, SIGNAL(copyPressed()), SLOT(copyPassword()));
     mx.connect(this, SIGNAL(downPressed()), SLOT(setFocus()));
+    mx.connect(SIGNAL(clearSearch()), m_ui->searchEdit, SLOT(clear()));
     mx.connect(m_ui->searchEdit, SIGNAL(returnPressed()), SLOT(switchToEntryEdit()));
 }
 
@@ -120,7 +136,7 @@ void SearchWidget::databaseChanged(DatabaseWidget* dbWidget)
         // Set current search text from this database
         m_ui->searchEdit->setText(dbWidget->getCurrentSearch());
         // Keyboard focus on search widget at database unlocking
-        connect(dbWidget, SIGNAL(unlockedDatabase()), this, SLOT(searchFocus()));
+        connect(dbWidget, SIGNAL(databaseUnlocked()), this, SLOT(searchFocus()));
         // Enforce search policy
         emit caseSensitiveChanged(m_actionCaseSensitive->isChecked());
         emit limitGroupChanged(m_actionLimitGroup->isChecked());
@@ -134,7 +150,7 @@ void SearchWidget::startSearchTimer()
     if (!m_searchTimer->isActive()) {
         m_searchTimer->stop();
     }
-    m_searchTimer->start(100);
+    m_searchTimer->start(300);
 }
 
 void SearchWidget::startSearch()
@@ -172,9 +188,17 @@ void SearchWidget::setLimitGroup(bool state)
     updateLimitGroup();
 }
 
-
 void SearchWidget::searchFocus()
 {
     m_ui->searchEdit->setFocus();
     m_ui->searchEdit->selectAll();
 }
+
+void SearchWidget::toggleHelp()
+{
+    if (m_helpWidget->isVisible()) {
+        m_helpWidget->hide();
+    } else {
+        m_helpWidget->show();
+    }
+}
